
WITH TEXT_IO; USE TEXT_IO;
WITH SEQUENTIAL_IO;



PROCEDURE agencia_matrimonial IS



------------------------------------------------------------------------------
--                     DECLARACION DE EXCEPCIONES                           --
------------------------------------------------------------------------------



no_cumple,fichero_vacio,DATA_ERROR,NAME_ERROR,STATUS_ERROR : EXCEPTION;



------------------------------------------------------------------------------
--                       DECLARACION DE TIPOS                               --
------------------------------------------------------------------------------



TYPE gustos IS RECORD
         cine,lectura,musica,deportes,juergas,hogar,naturaleza : INTEGER;
    END RECORD;     
    
TYPE sexos IS (H,V);

TYPE tabla_elejidos_ya IS ARRAY (1..50) OF INTEGER;

TYPE fichas IS RECORD
          numreg : INTEGER;
          nombre : STRING(1..15);         
          apels : STRING(1..30);                    
          direc : STRING(1..30);
          sexo : SEXOS;
          edad : INTEGER;
          gusto : GUSTOS;
     END RECORD;


     
------------------------------------------------------------------------------
--   DECLARACION DEL PAQUETE DE ENTRADAS/SALIDAS PARA EL REGISTRO FICHAS    --
------------------------------------------------------------------------------



PACKAGE ficha_es IS NEW SEQUENTIAL_IO(fichas); USE ficha_es;



------------------------------------------------------------------------------
--                          DECLARACION DE FICHEROS                         --
------------------------------------------------------------------------------



candida,solicitud : ficha_es.FILE_TYPE;
f3 : TEXT_IO.FILE_TYPE;



------------------------------------------------------------------------------
--                         DECLARACION DE VARIABLES                         --
------------------------------------------------------------------------------



tab : tabla_elejidos_ya;
postab : INTEGER;
filesal : STRING(1..12);
salida : CHARACTER;



------------------------------------------------------------------------------
--                   DECLARACION DE PROCEDIMIENTOS                          --
------------------------------------------------------------------------------



PROCEDURE inicializar_tabla(tbl : OUT tabla_elejidos_ya) IS
-- Entrada: Tabla de numeros enteros de 50 posiciones.
-- Salida : Misma tabla, pero inicializada con el valor cero. 
BEGIN
     FOR n IN 1..50 LOOP
         tbl(n) := 0;
     END LOOP;
     postab := 1;
END inicializar_tabla;         



FUNCTION mirar_si_ya_elejido(nreg : INTEGER) RETURN BOOLEAN IS
-- Entrada: Numero de registro leido previamente de un fichero.
-- Salida:  True or false dependiendo de si habia sido elejido ya o no.
BEGIN
     FOR n IN 1..postab LOOP
         IF tab(n) = nreg THEN 
             RETURN true;
         END IF;
     END LOOP;
     RETURN false;
END mirar_si_ya_elejido;         



FUNCTION puntos_candidato_solicitante(can,sol : fichas) RETURN INTEGER IS
-- Entrada: Dos registros uno de "CANDIDA.DAT" y otro de "SOLICI.DAT".
-- Salida:  Numero de puntos correspondiente al emparejamiento, para ello
--          se calculara la suma de la diferencia al cuadrado de cada uno
--          de los campos de los registros.
suma : INTEGER := 0;
BEGIN
     suma := (can.gusto.cine - sol.gusto.cine)**2;
     suma := (can.gusto.lectura - sol.gusto.lectura)**2 + suma;
     suma := (can.gusto.musica - sol.gusto.musica)**2 + suma;
     suma := (can.gusto.deportes - sol.gusto.deportes)**2 + suma;
     suma := (can.gusto.juergas - sol.gusto.juergas)**2 + suma;
     suma := (can.gusto.hogar - sol.gusto.hogar)**2 + suma;
     suma := (can.gusto.naturaleza - sol.gusto.naturaleza)**2 + suma;
     RETURN suma;
END puntos_candidato_solicitante;



FUNCTION leer_y_elejir(can : fichas)  RETURN fichas IS
-- Entrada: Un registro del fichero "CANDIDA.DAT".
-- Salida:  El registro que mejor le vaya, dentro del fichero "SOLICI.DAT".
ficha,mejor_reg : fichas;
mejor : INTEGER := 32000;
puntos : INTEGER := 0;
BEGIN
     OPEN(solicitud,IN_FILE,"SOLICI.DAT");
     WHILE NOT END_OF_FILE(solicitud) LOOP
          READ(solicitud,ficha);
          puntos := puntos_candidato_solicitante(ficha,can);
          IF puntos < mejor THEN 
             IF NOT mirar_si_ya_elejido(ficha.numreg) THEN
                    mejor := puntos;
                    mejor_reg := ficha;
             END IF;       
          END IF;
     END LOOP;
     CLOSE(solicitud);
     tab(postab) := mejor_reg.numreg;
     postab := postab + 1;
     RETURN mejor_reg;
END leer_y_elejir;



PROCEDURE configurar_salida IS
-- Configura la salida del procedimiento por pantalla, fichero o ambos.
BEGIN
     NEW_LINE(2);
     LOOP
         PUT("La salida la desea por Fichero(F) o por Pantalla(P)  ");
         GET(salida);
         NEW_LINE(2);
         IF salida /= 'F' OR salida /= 'f' OR 
            salida /= 'P' OR salida /= 'p' THEN
                   PUT("ERROR: El caracter que ha introducido no es valido.");
                   NEW_LINE(2);
         END IF;          
         EXIT WHEN salida = 'F' OR salida = 'f' OR
                   salida = 'P' OR salida = 'p';
     END LOOP;
     NEW_LINE(2);
     IF salida = 'F' OR salida = 'f' THEN
        PUT("Intruduce nombre de archivo de salida: ");
        GET(filesal);
        CREATE(f3,OUT_FILE,filesal);
     END IF;
     NEW_LINE(2);
END configurar_salida;        



PROCEDURE procesar_registros IS
-- Procesa los registros de "CANDIDA.DAT" con todos los de "SOLICI.DAT".
elejido,can : fichas;
   PROCEDURE imprimir(reg1,reg2 : fichas) IS
   BEGIN
        IF salida = 'P' THEN
           PUT("     ");
           PUT(reg1.apels);
           PUT(reg1.nombre);
           NEW_LINE;
           PUT("     ");           
           PUT(reg2.apels);
           PUT(reg2.nombre);
           NEW_LINE(2);
           ELSE
               PUT("     ");           
               PUT(f3,reg1.apels);
               PUT(f3,reg1.nombre);
               NEW_LINE;
               PUT("     ");
               PUT(f3,reg2.apels);
               PUT(f3,reg2.nombre);
               NEW_LINE(f3,2);
        END IF;
   END imprimir;            
BEGIN   
      WHILE NOT END_OF_FILE(candida) LOOP
           READ(candida,can);
           elejido := leer_y_elejir(can);
           imprimir(elejido,can);
      END LOOP;      
END procesar_registros;



PROCEDURE comprobar_ficheros IS
-- Comprueba los ficheros "candida" y "solicitud" y mira si estan vacios o
-- si el fichero "solicitud" tiene mas de 50 registros.
registro : fichas;
n : INTEGER := 0;
car : CHARACTER;
BEGIN
     OPEN(candida,IN_FILE,"CANDIDA.DAT");
     WHILE NOT END_OF_FILE(candida) LOOP
           READ(candida,registro);
           n := n + 1;
     END LOOP;
     CLOSE(candida);
     IF n > 50 THEN
        NEW_LINE;
        PUT("ATENCION: El fichero CANDIDA.DAT tiene mas de 50 registros.");
        NEW_LINE(2);
        PUT("Pulse S si desea continuar con la ejecucion del programa   ");
        IF car /= 'S' or car /= 's' THEN
           RAISE(no_cumple);
        END IF;
        ELSIF n - 1 = 0 THEN
              RAISE fichero_vacio;
     END IF;       
     N := 0;
     OPEN(solicitud,IN_FILE,"SOLICI.DAT");
     WHILE NOT END_OF_FILE(candida) LOOP
           READ(solicitud,registro);
           n := n + 1;
     END LOOP;
     CLOSE(solicitud);
     IF N -1 = 0 THEN 
        RAISE fichero_vacio;
     END IF;
     NEW_LINE(2);
END comprobar_ficheros;           



PROCEDURE cerrar_ficheros IS
-- Entrada: Ficheros a cerrar.
-- Salida : Ficheros cerrados.
BEGIN
     CLOSE(candida);
     IF salida = 'f' OR salida ='F' THEN
        CLOSE(f3);
     END IF;
END cerrar_ficheros;        



-------------------------------------------------------------------------------
--                DECLARACION DEL PROCEDIMIENTO PRINCIPAL                    --
-------------------------------------------------------------------------------




BEGIN



     comprobar_ficheros;
     inicializar_tabla(tab);
     OPEN(candida,IN_FILE,"CANDIDA.DAT");
     procesar_registros;
     cerrar_ficheros;

     
     EXCEPTION

     
      WHEN no_cumple =>
           NEW_LINE(2);
           PUT("ERROR: La ejecucion del programa ha sido suspendida debido a");
           NEW_LINE;
           PUT("       que el archivo CANDIDA.DAT tiene mas registros de los");
           NEW_LINE;
           PUT("       preestablecidos.");
           NEW_LINE(2);
           cerrar_ficheros;


      WHEN fichero_vacio =>
           NEW_LINE(2);
           PUT("ERROR: La ejecucion del programa ha sido suspendida debido a");
           NEW_LINE;
           PUT("       a que alguno de los ficheros de entrada esta vacio.");
           NEW_LINE(2);
           cerrar_ficheros;

     
      WHEN NAME_ERROR =>
           NEW_LINE(2);
           PUT("ERROR: La ejecucion del programa ha sido suspendida debido a");
           NEW_LINE;
           PUT("       que alguno de los ficheros de entrada no existe.");
           NEW_LINE(2);
           cerrar_ficheros;


      WHEN DATA_ERROR => 
           NEW_LINE(2);
           PUT("ERROR: La ejecucion del programa ha sido suspendida debido a");
           NEW_LINE;
           PUT("       que el tipo de datos en algun fichero es incompatible.");
           NEW_LINE(2);
           cerrar_ficheros;

              
      WHEN STATUS_ERROR =>
           NEW_LINE(2);
           PUT("ERROR: La ejecucion del programa ha sido suspendida debido a");
           NEW_LINE;
           PUT("       que se ha detectado un error en el estado de alguno de");
           NEW_LINE;
           PUT("       los ficheros de entrada.");
           NEW_LINE(2);
           cerrar_ficheros;



END agencia_matrimonial;

