

WITH TEXT_IO; USE TEXT_IO;


PROCEDURE CODIDECO IS

-- ENTRADA: FICHERO DE TEXTO.
-- SALIDA : TEXTO CODIFICADO Y POSICION DE SEPARADORES 
--          ELECCION ENTRE PANTALLA O FICHERO. 


-------------------------------------------------------------------------------
--                    DECLARACION DE EXCEPCIONES                             --
-------------------------------------------------------------------------------


DEM_LET,DEM_PAL,DEM_SEP,CAR_ERRON,FICH_VACIO : EXCEPTION; 


-------------------------------------------------------------------------------
--                DECLARACION DEL PAQUETE I/O PARA ENTEROS                   --
-------------------------------------------------------------------------------


PACKAGE ES_INTEGER IS
        NEW INTEGER_IO (INTEGER);
USE ES_INTEGER;


-------------------------------------------------------------------------------
--                   DECLARACION DE CONSTANTES                               --
-------------------------------------------------------------------------------


MAXIMO_PALABRAS : CONSTANT NATURAL := 100;


-------------------------------------------------------------------------------
--                        DECLARACION DE SUBTIPOS                            --
-------------------------------------------------------------------------------


SUBTYPE PALABRA IS STRING(1..25);
SUBTYPE ORDEN IS NATURAL RANGE 0..129;
SUBTYPE FILE IS STRING(1..12);


-------------------------------------------------------------------------------
--                         DECLARACION DE TIPOS                              --
-------------------------------------------------------------------------------


TYPE TIPOS IS (MAYUSCULA,MINUSCULA);
TYPE SEPARADORES IS (PUNTO,COMA,NINGUNO);
TYPE LISTA_PALABRAS IS ARRAY (1..MAXIMO_PALABRAS) OF PALABRA;
TYPE POSICION_SEPARADOR IS 
    RECORD
          POSICION : ORDEN;
          SEPARADOR : SEPARADORES;
    END RECORD;
TYPE PUNTOS_COMAS IS ARRAY (1..30) OF POSICION_SEPARADOR;
TYPE POSICIONES IS
    RECORD
          NUMERO_SEPARADORES : NATURAL RANGE 0..30;
          TABLA_SEPARADORES : PUNTOS_COMAS;
    END RECORD;
TYPE POS_FIN_LINEAS IS ARRAY (1..10) OF INTEGER;


-------------------------------------------------------------------------------
--                        DECLARACION DE VARIABLES                           --
-------------------------------------------------------------------------------


LISTCOD : LISTA_PALABRAS;    
POSLIN : POS_FIN_LINEAS;
NPA : INTEGER := 1;
L : SEPARADORES;
POSIC : POSICIONES;
CLAVE : INTEGER;
N,M : INTEGER := 0;
FICHERO : BOOLEAN;


-------------------------------------------------------------------------------
--                         DECLARACION DE FICHEROS                           --
-------------------------------------------------------------------------------


F1,F2 : FILE_TYPE;
FILE1,FILE2 : FILE;


-------------------------------------------------------------------------------
--                         DECLARACION DE PROCEDIMIENTOS                     --
-------------------------------------------------------------------------------



PROCEDURE INICIALIZAR(LISTA : OUT LISTA_PALABRAS; POS : OUT POSICIONES;
                      PLIN : OUT POS_FIN_LINEAS) IS
-- ENTRADA: TABLA DE PALABRAS, TABLA DE SEPARADORES, TABLA DE FIN DE LINEAS.
-- SALIDA:  TABLA DE PALABRAS INICIALIZADA A ESPACIOS
--          TABLA DE SEPARADORES A CERO
--          TABLA DE FIN DE LINEAS INICIALIZADA A CERO.
BEGIN
     FOR X IN 1..MAXIMO_PALABRAS LOOP
         LISTA(X) := "                         ";
     END LOOP;
     POS.NUMERO_SEPARADORES := 0;
     FOR X IN 1..10 LOOP
         PLIN(X) := 0;
     END LOOP;
     FOR X IN 1..30 LOOP
         POSIC.TABLA_SEPARADORES(X).SEPARADOR := NINGUNO;
     END LOOP;    
END INICIALIZAR;



FUNCTION CODIFICAR_PALABRA( PAL : IN PALABRA ) RETURN PALABRA IS
-- ENTRADA: PALABRA A CODIFICAR.
-- SALIDA : PALABRA CODIFICADA.
PALAB : PALABRA;
  
  FUNCTION CODIFICA_LETRA( LET : IN CHARACTER ) RETURN CHARACTER IS
  -- ENTRADA: LETRA A CODIFICAR.
  -- SALIDA : LETRA CODIFICADA.
  POS : INTEGER;
  
    FUNCTION CODIFICA( L : IN CHARACTER ) RETURN CHARACTER IS
    -- ENTRADA: LETRA A CODIFICAR
    -- SALIDA : LETRA CODIFICADA
    I : INTEGER;
    PRIN : INTEGER;
    FIN : INTEGER;
    BEGIN
         IF L = ' ' THEN RETURN L; END IF;
         I := CHARACTER'POS(L);
         IF (I >=65) AND (I <=90) THEN PRIN := 65; FIN := 90; END IF;
         IF (I >=97) AND (I <=122) THEN PRIN := 97; FIN := 122; END IF;
         IF CLAVE > 0 THEN
            IF I + CLAVE <= FIN THEN
               I := I + CLAVE;
               ELSE
                   I := I + CLAVE - 26;
            END IF;
         END IF;   
         IF CLAVE < 0 THEN
            IF I + CLAVE >= PRIN THEN
               I := I + CLAVE;
               ELSE
                   I := I + CLAVE + 26;
            END IF;       
         END IF;   
         RETURN CHARACTER'VAL(I);
    END CODIFICA;         
    
  BEGIN
       RETURN CODIFICA(LET);
  END CODIFICA_LETRA;
  
BEGIN
     PALAB := PAL;
     FOR X IN 1..25 LOOP
         PALAB(X) := CODIFICA_LETRA(PAL(X));
     END LOOP;
     RETURN PALAB;
END CODIFICAR_PALABRA;



PROCEDURE SEPARADOR(LET : CHARACTER) IS
-- ENTRADA: SEPARADOR, PUNTO O COMA.
-- SALIDA : UBICACION EN LA TABLA, TIPO DE SEPARADOR Y POSICION EN TEXTO. 
BEGIN
     IF POSIC.NUMERO_SEPARADORES = 30 THEN
        RAISE DEM_SEP;
        ELSE
            IF LET = '.' THEN 
               L := PUNTO;
            END IF;
            IF LET = ',' THEN
               L := COMA;
            END IF;   
            NPA := NPA + 1;
            POSIC.NUMERO_SEPARADORES := POSIC.NUMERO_SEPARADORES + 1;
            POSIC.TABLA_SEPARADORES(POSIC.NUMERO_SEPARADORES).POSICION := NPA;
            POSIC.TABLA_SEPARADORES(POSIC.NUMERO_SEPARADORES).SEPARADOR := L;
     END IF;
END SEPARADOR;


FUNCTION FIN_LINEA(TAB : POS_FIN_LINEAS; NPAL : INTEGER) RETURN BOOLEAN IS
-- ENTRADA: TABLA FORMADA POR POSICIONES DE FIN DE LINEA
--          NUMERO DE PALABRA EN EL QUE VA.
-- SALIDA:  TRUE O FALSE DEPENDIENDO DE SI EL NUMERO DE PALABRA EN EL QUE VA
--          CORRESPONDE CON UN FIN DE LINEA.
BEGIN
     FOR N IN 1..10 LOOP
         IF TAB(N) = NPAL THEN
            RETURN TRUE;
         END IF;          
     END LOOP;
     RETURN FALSE;
END FIN_LINEA;         

PROCEDURE PR_FICH(TXT : PALABRA) IS
-- ENTRADA: PALABRA CODIFICADA.
-- SALIDA : PALABRA CODIFICADA IMPRESA POR FICHERO.
N : INTEGER := 1;
BEGIN
     WHILE TXT(N) /= ' ' AND N < 25 LOOP
           PUT(F2,TXT(N));
           N := N + 1;
     END LOOP;      
END PR_FICH;     

PROCEDURE PR_PANT(TXT : PALABRA) IS
-- ENTRADA: PALABRA CODIFICADA.
-- SALIDA : PALABRA CODIFICADA IMPRESA POR PANTALLA.
N : INTEGER := 1;
BEGIN
     WHILE TXT(N) /= ' ' AND N < 25 LOOP
           PUT(TXT(N));
           N := N + 1;
     END LOOP;
END PR_PANT;           

PROCEDURE IMPRIME1(LISTA : IN OUT LISTA_PALABRAS; POS : IN OUT POSICIONES) IS
-- ENTRADA: TABLA FORMADA POR PALABRAS CODIFICADAS, TABLA DE SEPARADORES.
-- SALIDA : IMPRESION DE TODO POR PANTALLA, O POR FICHERO.
PACKAGE ES_SEPARADORES IS NEW ENUMERATION_IO (SEPARADORES);
USE ES_SEPARADORES;
X : INTEGER := 0;
    
    PROCEDURE PR_SEPAR IS
    -- ENTRADA: TABLA DE SEPARADORES.
    -- SALIDA : TIPO DE SEPARADOR Y SU POSICION, POR PANTALLA O POR FICHERO.
    N : INTEGER := 1;
    BEGIN
         IF NOT FICHERO THEN 
            NEW_LINE(2);
            PUT("Punto en las posiciones: ");
            WHILE N <= POS.NUMERO_SEPARADORES OR N = 30 LOOP
                IF POS.TABLA_SEPARADORES(N).SEPARADOR = PUNTO THEN
                   PUT(POS.TABLA_SEPARADORES(N).POSICION);
                END IF;
                N := N + 1;
            END LOOP;
            NEW_LINE;    
            N := 1;
            PUT("Coma  en las posiciones: ");
            WHILE N <= POS.NUMERO_SEPARADORES OR N = 30 LOOP
                IF POS.TABLA_SEPARADORES(N).SEPARADOR = COMA THEN
                   PUT(POS.TABLA_SEPARADORES(N).POSICION);
                END IF;
                N := N + 1;
            END LOOP;
            NEW_LINE;      
            ELSIF FICHERO THEN
                 NEW_LINE(F2,2);
                 PUT(F2,"Punto en las posiciones: ");
                 WHILE N <= POS.NUMERO_SEPARADORES OR N = 30 LOOP
                     IF POS.TABLA_SEPARADORES(N).SEPARADOR = PUNTO THEN
                        PUT(F2,POS.TABLA_SEPARADORES(N).POSICION);
                     END IF;
                     N := N + 1;
                 END LOOP;
                 NEW_LINE(F2);
                 N := 1;
                 PUT(F2,"Coma en las posiciones:  ");
                 WHILE N <= POS.NUMERO_SEPARADORES OR N = 30 LOOP
                     IF POS.TABLA_SEPARADORES(N).SEPARADOR = COMA THEN
                        PUT(F2,POS.TABLA_SEPARADORES(N).POSICION);
                     END IF;
                     N := N + 1;          
                 END LOOP;    
                 NEW_LINE(F2);
            END IF;       
    END PR_SEPAR;     
    
BEGIN
     IF FICHERO = TRUE THEN
        NEW_LINE(F2);
     END IF;   
     LOOP
         X := X + 1;
         IF FICHERO = TRUE THEN
            IF FIN_LINEA(POSLIN,X) THEN 
               NEW_LINE(F2);
            END IF;   
            PR_FICH(LISTA(X));
            PUT(F2,' ');
            ELSE
                IF FIN_LINEA(POSLIN,X) THEN
                   NEW_LINE;
                END IF;   
                PR_PANT(LISTA(X));
                PUT(' ');
         END IF;       
         EXIT WHEN X > NPA OR X = 100;
     END LOOP;
     IF POS.NUMERO_SEPARADORES = 0 THEN
        IF NOT FICHERO THEN
           NEW_LINE(2);
           PUT("No hay separadores en el texto.");
           NEW_LINE(2);
           ELSIF FICHERO THEN
                 NEW_LINE(F2,2);
                 PUT(F2,"No hay separadores en el texto.");
                 NEW_LINE(F2,2);
        END IF;
        ELSE         
           PR_SEPAR;
     END IF;   
     IF FICHERO THEN
        NEW_LINE(F2,3);
        ELSE
            NEW_LINE(3);
     END IF;       
END IMPRIME1;         

PROCEDURE IMPRIME2(LISTA : IN LISTA_PALABRAS; POS : IN POSICIONES) IS
-- ENTRADA: TABLA FORMADA POR PALABRAS DECODIFICADAS, TABLA DE SEPARADORES.
-- SALIDA : IMPRESION DEL TEXTO POR PANTALLA O FICHERO.
N,NP : INTEGER := 1;
    FUNCTION BUSCAR_SEPARADOR(NS : IN INTEGER) RETURN SEPARADORES IS
    -- ENTRADA: NUMERO DE POSICION EN TEXTO.
    -- SALIDA : COMA, PUNTO O NINGUNO DEPENDIENDO DE LO QUE HAYA EN ESA
    --          POSICION DE PALABRA.
    BEGIN
         FOR X IN 1..30 LOOP
             IF POS.TABLA_SEPARADORES(X).POSICION = NS THEN
                IF POS.TABLA_SEPARADORES(X).SEPARADOR = COMA THEN
                   RETURN COMA; 
                   ELSIF POS.TABLA_SEPARADORES(X).SEPARADOR = PUNTO THEN 
                         RETURN PUNTO; 
                END IF;
             END IF;
         END LOOP;
         RETURN NINGUNO;
    END BUSCAR_SEPARADOR;                     
               
BEGIN             
     WHILE LISTA(N) /= "                         " LOOP
           IF BUSCAR_SEPARADOR(NP) = PUNTO THEN 
              IF FICHERO THEN
                 PUT(F2,'.');
                 ELSE
                     PUT('.');
              END IF;       
              NP := NP + 1;
              ELSIF BUSCAR_SEPARADOR(NP) = COMA THEN
                    IF FICHERO THEN
                       PUT(F2,',');
                       ELSE
                           PUT(',');
                    END IF;       
                    NP := NP + 1;
           END IF;
           IF FIN_LINEA(POSLIN,N) THEN
              IF FICHERO THEN
                 NEW_LINE(F2);
                 ELSE
                     NEW_LINE;
              END IF;       
           END IF;   
           IF N /= 1 AND NOT FIN_LINEA(POSLIN,N) THEN
              IF FICHERO THEN
                 PUT(F2,' ');
                 ELSE
                     PUT(' ');
              END IF;       
           END IF;
           IF FICHERO THEN
              PR_FICH(LISTA(N));
              ELSE
                  PR_PANT(LISTA(N));
           END IF;       
           N := N + 1;
           NP := NP + 1;
           EXIT WHEN N = 101;
     END LOOP;
     LOOP
        IF FIN_LINEA(POSLIN,NP) THEN
           IF FICHERO THEN
              NEW_LINE(F2);
              ELSE
                  NEW_LINE;
           END IF;
        END IF;          
        IF BUSCAR_SEPARADOR(NP) = PUNTO THEN
           IF FICHERO THEN
              PUT(F2,'.');
              PUT(F2,' ');
              ELSE
                  PUT('.');
                  PUT(' ');
           END IF;       
           ELSIF BUSCAR_SEPARADOR(NP) = COMA THEN
                 IF FICHERO THEN
                    PUT(F2,',');
                    PUT(F2,' ');
                    ELSE
                        PUT(',');
                        PUT(' ');
                 END IF;       
        END IF;
        NP := NP + 1;
        EXIT WHEN NP >= 130;
     END LOOP;   
     IF FICHERO THEN
        NEW_LINE(F2,3);
        ELSE
            NEW_LINE(3);
     END IF;       
END IMPRIME2;           


FUNCTION COMP_CARAC(LE : CHARACTER) RETURN BOOLEAN IS
-- ENTRADA: UNA LETRA.
-- SALIDA : TRUE O FALSE DEPENDIENDO DE SI ES CARACTER PERMITIDO O NO. 
L : INTEGER;
BEGIN
     L := CHARACTER'POS(LE);
     IF L >=65 AND L <=90 THEN
        RETURN TRUE;
        ELSIF L >=91 AND L <=122 THEN
              RETURN TRUE;
              ELSIF LE = ',' OR LE = '.' OR LE = ' ' THEN
                    RETURN TRUE;
                    ELSE
                        RETURN FALSE;
     END IF;                   
END COMP_CARAC;      



FUNCTION LEER_PALABRA(F : FILE_TYPE) RETURN PALABRA IS
-- ENTRADA: FICHERO DE TEXTO.
-- SALIDA : UNA PALABRA DEL FICHERO 
--          SEPARADOR EN TABLA, SI SE TERCIA.  
BUFFER : PALABRA;
C : CHARACTER := ' ';
T : INTEGER;
BEGIN
     FOR T IN 1..25 LOOP BUFFER(T) := ' '; END LOOP;
     T := 0;
     LOOP
         IF (NOT END_OF_FILE(F)) OR (NOT END_OF_LINE(F)) THEN
            GET(F,C);
         END IF;
         IF NOT COMP_CARAC(C) THEN
            RAISE CAR_ERRON;
         END IF;   
         IF C /= ',' AND C /= '.' AND C /= ' ' THEN
            T:=T+1;
            IF T >= 26 THEN
               RAISE DEM_LET;
            END IF;   
            BUFFER(T):= C;
         END IF;
         IF (C = ',') OR (C = '.') THEN
            SEPARADOR(C);
         END IF;
         EXIT WHEN C = ' ' OR END_OF_FILE(F) OR END_OF_LINE(F);
     END LOOP;
     NPA := NPA + 1;
     RETURN BUFFER;
END LEER_PALABRA;



PROCEDURE CONFIGURAR(CLV : IN OUT INTEGER; FIL1,FIL2 : IN OUT FILE) IS
-- PETICION DE CLAVE Y FICHEROS DESDE TECLADO.
X : CHARACTER;

    FUNCTION COMP_VACIA RETURN BOOLEAN IS
    -- ENTRADA: FICHERO DE TEXTO DE ENTRADA.
    -- SALIDA : TRUE SI FICHERO VACIO
    --          FALSE SI FICHERO NO VACIO.
    X : CHARACTER;
    BEGIN
         OPEN(F1,IN_FILE,FILE1);
         LOOP
             IF END_OF_FILE(F1) THEN 
                CLOSE(F1);
                RETURN TRUE;
             END IF;   
             GET(F1,X);
             EXIT WHEN X /= ' ' OR END_OF_FILE(F1);
         END LOOP;
         IF END_OF_FILE(F1) THEN
            CLOSE(F1);
            RETURN TRUE;
         END IF;
         CLOSE(F1);
         RETURN FALSE;
    END COMP_VACIA;        
    
BEGIN
     NEW_LINE;
     LOOP
         PUT("Introduce la Clave: ");
         GET(CLV);
         NEW_LINE;
         IF CLV > 25 OR CLV < -25 THEN
            PUT("ERROR: Clave Fuera de Rango  -25 =< CLAVE =< 25");
            NEW_LINE(2);
         END IF;
         EXIT WHEN CLV <= 25 AND CLV >= -25;
     END LOOP;       
     PUT("Introduce Nombre de Archivo: ");
     GET(FIL1);
     IF COMP_VACIA THEN
        RAISE FICH_VACIO;
     END IF;   
     NEW_LINE;
     PUT("Por Pantalla: (S/N)  ");
     GET(X);
     NEW_LINE;
     IF X ='N' OR X = 'n' THEN
        FICHERO := TRUE;
        PUT("Introduce Nombre de Archivo para Texto de Salida: ");
        GET(FIL2);
        NEW_LINE(2);
        ELSE
            FICHERO := FALSE;
     END IF;       
     OPEN(F1,IN_FILE,FIL1);
     IF FICHERO THEN
        CREATE(F2,OUT_FILE,FIL2);
     END IF;   
END CONFIGURAR;     



PROCEDURE CERRAR_FICHEROS(FF1,FF2 : IN OUT FILE_TYPE) IS
-- ENTRADA: FICHEROS PARA CERRAR.
-- SALIDA : FICHEROS CERRADOS.
BEGIN
     CLOSE(FF1);
     IF FICHERO THEN
        CLOSE(FF2);
     END IF;
END CERRAR_FICHEROS;        



PROCEDURE PROCESAR IS
-- ENTRADA: FICHERO DE TEXTO.
-- SALIDA : PALABRAS CODIFICADAS Y ALMACENADAS EN TABLA
--          SEPARADORES ALMACENADOS EN TABLA.
BEGIN
     WHILE NOT END_OF_FILE(F1) LOOP
           IF END_OF_LINE(F1) THEN
              POSLIN(M) := N + 1;
              M := M + 1;
           END IF;
           N := N + 1;
           IF N > 100 THEN
              RAISE DEM_PAL;
           END IF;
           LISTCOD(N) := LEER_PALABRA(F1);
           LISTCOD(N) := CODIFICAR_PALABRA(LISTCOD(N));
     END LOOP;
END PROCESAR;



-------------------------------------------------------------------------------
--                       PROCEDIMIENTO PRINCIPAL                             --
-------------------------------------------------------------------------------



BEGIN

     M := 1;
     INICIALIZAR(LISTCOD,POSIC,POSLIN);
     CONFIGURAR(CLAVE,FILE1,FILE2);
     PROCESAR;
     IMPRIME1(LISTCOD,POSIC);
     CLAVE := -(CLAVE);
     FOR N IN 1..100 LOOP
         LISTCOD(N) := CODIFICAR_PALABRA(LISTCOD(N));    
     END LOOP;
     IMPRIME2(LISTCOD,POSIC);    
     CERRAR_FICHEROS(F1,F2);
     
-- DECLARACION DE ACCIONES DE ERROR DE LAS EXCEPCIONES.     

     EXCEPTION 
     
        WHEN DEM_SEP => 
          NEW_LINE(2);      
          PUT("ERROR: El texto no cumple con las condiciones exigidas. ");
          NEW_LINE;
          PUT("Numero maximo de palabras = 100");
          NEW_LINE;
          PUT("Numero maximo de separadores = 25  <===> ERROR");
          NEW_LINE;
          PUT("Longitud maxima por palabra (caracteres) = 25");
          NEW_LINE;
          PUT("Caracteres validos: A..Z  a..z  , . ");
          NEW_LINE;
          CERRAR_FICHEROS(F1,F2);

        WHEN CAR_ERRON =>
          NEW_LINE(2);
          PUT("ERROR: El texto no cumple con las condiciones exigidas. ");
          NEW_LINE;
          PUT("Numero maximo de palabras = 100");
          NEW_LINE;
          PUT("Numero maximo de separadores = 25");
          NEW_LINE;
          PUT("Longitud maxima por palabra (caracteres) = 25");
          NEW_LINE;
          PUT("Caracteres validos: A..Z a..z , .  <===> ERROR");
          NEW_LINE;
          CERRAR_FICHEROS(F1,F2);

        WHEN DEM_PAL =>
          NEW_LINE(2);
          PUT("ERROR: El texto no cumple con las condiciones exigidas.");
          NEW_LINE;
          PUT("Numero maximo de palabras = 100  <===> ERROR");
          NEW_LINE;
          PUT("Numero maximo de separadores = 25");
          NEW_LINE;
          PUT("Longitud maxima por palabra (caracteres) = 25");
          NEW_LINE;
          PUT("Caracteres validos: A..Z a..z , .  <===> ERROR");
          NEW_LINE;
          CERRAR_FICHEROS(F1,F2);

        WHEN DEM_LET =>
          NEW_LINE(2);
          PUT("ERROR: El texto no cumple con las condiciones exigidas.");
          NEW_LINE;
          PUT("Numero maximo de palabras = 100");
          NEW_LINE;
          PUT("Numero maximo de separadores = 25");
          NEW_LINE;
          PUT("Longitud maxima por palabra (caracteres) = 25  <===> ERROR");
          NEW_LINE;
          PUT("Caracteres validos: A..Z a..z , .  <===> ERROR");
          NEW_LINE;
          CERRAR_FICHEROS(F1,F2);
          
        WHEN FICH_VACIO =>
          NEW_LINE;
          PUT("ERROR: El fichero de entrada esta vacio.");  
          NEW_LINE;
          
        WHEN NAME_ERROR =>
          NEW_LINE;
          PUT("ERROR: El fichero que ha sido introducido no existe.");
          NEW_LINE;
          
        WHEN STATUS_ERROR =>
          NEW_LINE;
          PUT("ERROR: Se ha detectado una anomalia en el estado del fichero.");
          NEW_LINE;
          CERRAR_FICHEROS(F1,F2);
          
        WHEN OTHERS =>
          NEW_LINE;
          PUT("Se ha producido un error sin determinar.");
          NEW_LINE;
          PUT("Posiblemente sea un error de Entrada/Salida.");
          NEW_LINE;

               
END CODIDECO;
