
WITH TEXT_IO; USE TEXT_IO;

PROCEDURE ECUATION IS

-- RESOLUCION DE UN SISTEMA DE N ECUACIONES CON N INCOGNITAS.

PACKAGE ES_FLOAT IS NEW FLOAT_IO(FLOAT); USE ES_FLOAT;
PACKAGE ES_INTEGER IS NEW INTEGER_IO(INTEGER); USE ES_INTEGER;

MAXSIZE : CONSTANT INTEGER := 100;
SUBTYPE INDEX IS INTEGER RANGE 1..MAXSIZE;
TYPE RMATRIX IS ARRAY(INDEX,INDEX) OF FLOAT;
TYPE RARRAY IS ARRAY(INDEX) OF FLOAT;
TYPE IARRAY IS ARRAY(INDEX) OF INTEGER;
SUB : IARRAY;
SIZE : INTEGER;
A : RMATRIX;
X,B : RARRAY;
T : FLOAT;


PROCEDURE LU_FACTOR(N : IN INTEGER; A: IN OUT RMATRIX) IS
INDX ,J : INTEGER;
PIVOT,MAX,AB,T : FLOAT;

   PROCEDURE ERROR IS
   BEGIN
        NEW_LINE(2);
        PUT(" La matriz de coeficientes es singular");
        NEW_LINE;
        PUT(" No puede continuarse el procesamiento");
        NEW_LINE(2);
   END ERROR;
   
BEGIN
     IF N > MAXSIZE THEN
        NEW_LINE;
        PUT(" Puede resolverse solo hasta");
        PUT(MAXSIZE);
        PUT("ecuaciones simultaneas");
        NEW_LINE(2);
     ELSE
        FOR I IN 1..N LOOP
            SUB(I) := I;
        END LOOP;
        FOR K IN 1..N-1 LOOP
            MAX := 0.0;
            FOR I IN K..N LOOP
                T := A(SUB(I),K);
                AB := ABS(T);
                IF AB > MAX THEN
                   MAX := AB;
                   INDX := I;
                END IF;
            END LOOP;
            IF MAX <= 1.0E-7 THEN
               ERROR;
            END IF;
            J := SUB(K);
            SUB(K) := SUB(INDX);
            SUB(INDX) := J;
            PIVOT := A(SUB(K),K);
            FOR I IN K+1..N LOOP
                A(SUB(I),K) := A(SUB(I),K) / PIVOT;
                FOR J IN K+1..N LOOP
                    A(SUB(I),J) := A(SUB(I),J) + A(SUB(I),K) * A(SUB(K),J);
                END LOOP;
            END LOOP;
        END LOOP;
        FOR I IN 1..N LOOP
            IF A(SUB(I),I) = 0.0 THEN
               ERROR;
            END IF;
        END LOOP;
     END IF;
END LU_FACTOR;                
        

PROCEDURE SOLVE(N : IN INTEGER; A: IN RMATRIX; C: IN RARRAY; X: IN OUT RARRAY) IS
K : INTEGER;
BEGIN
     IF N = 1 THEN
        X(1) := C(1)/A(1,1);
     ELSE
        X(1) := C(SUB(1));
        FOR K IN 2..N LOOP
            X(K) := C(SUB(K));
            FOR I IN 1..K-1 LOOP
                X(K) := X(K) + A(SUB(K),I)*X(I);
            END LOOP;
        END LOOP;
        X(N) := X(N) / A(SUB(N),N);        
        K := N;
        LOOP
            K := K - 1;
            FOR I IN K+1..N LOOP
                X(K) := X(K) - A(SUB(K),I) * X(I);
            END LOOP;
            X(K) := X(K) / A(SUB(K),K);
            EXIT WHEN K = 1;
        END LOOP;
     END IF;
END SOLVE;


PROCEDURE MESSAGE IS
BEGIN
     NEW_LINE;
     PUT(" Se resuelven N ecuaciones simultaneas de la forma");
     NEW_LINE(2);
     PUT("            A*X = B");
     NEW_LINE(2);
     PUT(" donde A es una matriz n x n de coeficiente, y B");
     NEW_LINE;
     PUT(" es un vector de coeficientes en columna.");
     NEW_LINE(3);
END MESSAGE;


PROCEDURE INPUT IS
BEGIN
     PUT(" Ingresar el numero de ecuaciones que se resolveran: ");
     GET(SIZE);
     NEW_LINE;
     IF SIZE > MAXSIZE THEN
        NEW_LINE;
        PUT(" No se pueden resolver mas de");
        PUT(MAXSIZE);
        PUT(" ecuaciones.");
        NEW_LINE(2);
        INPUT;
     END IF;
     NEW_LINE;
     FOR I IN 1..SIZE LOOP
         FOR J IN 1..SIZE LOOP
             PUT(" Ingresar A("); 
             PUT(I);
             PUT(".");
             PUT(J);
             PUT("): ");
             GET(T);
             A(I,J) := T;
         END LOOP;
     END LOOP;
     NEW_LINE;
     FOR K IN 1..SIZE LOOP
         PUT(" Ingresar B(");
         PUT(K);
         PUT("): "); 
         GET(T);
         B(K) := T;
     END LOOP;
END INPUT;


PROCEDURE OUTPUT IS
T : FLOAT;
BEGIN
     NEW_LINE;
     PUT(" La solucion");
     NEW_LINE;
     PUT(" ===========");
     NEW_LINE(2);
     FOR I IN 1..SIZE LOOP
         T := X(I);
         PUT(T);
         NEW_LINE;
     END LOOP;
END OUTPUT;


BEGIN

     MESSAGE;
     INPUT;
     LU_FACTOR(SIZE,A);
     SOLVE(SIZE,A,B,X);
     OUTPUT;

END ECUATION;

