with Tokens,
     Lista_Palabras_Reservadas;

package body Analexico is

   Blanco : constant Character := ' ';
   Comillas : constant Character := ASCII.Quotation;
   Ampersand : constant Character := '&';
   Subrayado : constant Character := '_';
   Guion : constant Character := '-';
   Tabulador : constant Character := ASCII.HT;
   Fin_Linea : constant Character := ASCII.CR;
   Apostrofo : constant Character := ''';

   Num_Estados_Max : constant Natural := 14;

   subtype Letra_May is Character range 'A' .. 'Z';
   subtype Letra_Min is Character range 'a' .. 'z';
   subtype Digito is Character range '0' .. '9';

   subtype Estado is Natural range 0 .. Num_Estados_Max;
   subtype Estado_No_Terminal is Natural range 1 .. Num_Estados_Max;

   type T_Transiciones is array (Estado_No_Terminal,
                                 Character) of Estado;

   Tabla_Transiciones : constant T_Transiciones :=
     (1 => (Blanco | Tabulador => 10,
            Fin_Linea => 11,
            Comillas => 3,
            Ampersand => 5,
            Letra_May | Letra_Min => 8,
            Guion => 6,
            Apostrofo => 12,
            others => 2),
      2 => (Blanco | Tabulador | Comillas | Ampersand |
            Fin_Linea | Letra_May | Letra_Min | Apostrofo |
            Guion => 0, -- token de tipo Otro
            others => 2),
      3 => (Comillas => 4,
            others => 3),
      4 => (others => 0), -- token Cadena
      5 => (others => 0), -- token Concatenador
      6 => (Guion => 7,
            others => 2),
      7 => (Fin_Linea => 0, -- token Comentario
            others => 7),
      8 => (Subrayado => 9,
            Letra_May | Letra_Min | Digito => 8,
            others => 0), -- token ident. o pal. reservada
      9 => (Letra_May | Letra_Min | Digito => 8,
            others => 0),
      -- aqui no es posible Fin_Linea ni ningun otro token
      -- en un program Ada sintacticamente correcto
      10 => (Blanco | Tabulador => 10,
             others => 0), -- token Secuencia_Blancos
      11 => (others => 0), -- token Fin_Linea
      12 => (others => 13),
      13 => (Apostrofo => 14,
             others => 0), -- Otro (apostrofo: hay que retroceder 1 posicion)
      14 => (others => 0)); -- token Otro (literal tipo caracter)

   No_Consumido : Strings_Variables.String_Variable;
   -- cuando se esta tratando una linea, el resto
   -- de linea sin consumir estara en esta variable del paquete

   function Terminal (E : in Estado) return Boolean is

   begin
      return (E = 2) or (E = 4) or (E = 5) or (E = 7)
        or (E = 8) or (E = 10) or (E = 11) or (E = 13) or (E = 14);
   end Terminal;

   function Token_Reconocido return Tokens.Token is
      ---------------------------------------------------
      -- Entrada    :
      -- Salida     : token reconocido
      -- Efecto     : recorre No_Consumido y devuelve
      --              el primer token reconocido
      -- Efectos laterales: No_Consumido se modifica (se elimina de su parte
      --                    frontal la parte consumida)
      -- Excepciones:
      -- Observaciones:
      ---------------------------------------------------

      function Siguiente (Pos_Sig_Car : in Positive;
                          S : in String) return Character is
      begin
         if Pos_Sig_Car > S'Last then
            return Fin_Linea;
         else
            return S (Pos_Sig_Car);
         end if;
      end Siguiente;

      Reconocido : Boolean := False;
      Pos_Sig_Car : Positive := 1;
      Estado_Anterior : Estado := 1;
      Estado_Actual : Estado;
      Caracter : Character;
   begin
      Caracter := Siguiente (Pos_Sig_Car,
                             Strings_Variables.Valor (No_Consumido));
      while not Reconocido loop
         Estado_Actual := Tabla_Transiciones (Estado_Anterior, Caracter);
         if (Estado_Actual = 0) and
            Terminal (Estado_Anterior) then
            Reconocido := True;
         else
            Pos_Sig_Car := Pos_Sig_Car + 1;
            Caracter := Siguiente (Pos_Sig_Car,
                                   Strings_Variables.Valor (No_Consumido));
            Estado_Anterior := Estado_Actual;
         end if;
      end loop;
      if Estado_Anterior = 13 then Pos_Sig_Car := Pos_Sig_Car - 1;
      end if; --se ha reconocido el apostrofo pero no es un literal caracter:
      -- para ello se ha avanzado una pos. en la cadena, por lo que hay que
      -- retroceder 1
      declare
         Cadena_Reconocida : String (1 .. Pos_Sig_Car - 1) :=
           Strings_Variables.Valor (No_Consumido) (1 .. Pos_Sig_Car - 1);
         Cadena_Token : Strings_Variables.String_Variable;
      begin
         Strings_Variables.Copiar
           (Cadena_Token, Strings_Variables.FString_Var (Cadena_Reconocida));
         Strings_Variables.Copiar
           (No_Consumido,
            Strings_Variables.FString_Var       -- efecto lateral
            (Strings_Variables.Valor (No_Consumido)
             (Pos_Sig_Car .. Strings_Variables.Longitud (No_Consumido))));
         case Estado_Anterior is
            when 2 | 13 | 14 =>
               return Tokens.FToken (Cadena_Token, Tokens.Otro);
            when 4 =>
               return Tokens.FToken (Cadena_Token, Tokens.Cadena);
            when 5 =>
               return Tokens.FToken (Cadena_Token, Tokens.Concatenador);
            when 7 =>
               return Tokens.FToken (Cadena_Token, Tokens.Comentario);
            when 8 => -- palabra reservada o identificador
               if Lista_Palabras_Reservadas.Esta (Cadena_Token) then
                  return
                    Tokens.FToken (Cadena_Token, Tokens.Palabra_Reservada);
               else
                  return
                    Tokens.FToken (Cadena_Token, Tokens.Identificador);
               end if;
            when 10 =>
               return Tokens.FToken (Cadena_Token, Tokens.Secuencia_Blancos);
            when 11 =>
               return Tokens.FToken (Cadena_Token, Tokens.Fin_Linea);
            when others =>
               raise Token_Desconocido;
            -- por ej. si el estado anterior es 9,
            -- lo que no es posible en un programa correcto
         end case;
      end;
   end Token_Reconocido;

   procedure Obtener_Lista_Apunt_Tokens
     (L : in Strings_Variables.String_Variable;
      LAT : out Listas_Apunt_Tokens.Tipo_Lista_Apunt_Tokens) is
      Apunt_T : Listas_Apunt_Tokens.Apunt_Token;
   begin
      Strings_Variables.Copiar
         (No_Consumido,
          Strings_Variables.FString_Var (Strings_Variables.Valor (L)));
      Listas_Apunt_Tokens.Crear_Lista_Vacia (LAT);
      while not Strings_Variables.Es_Vacio (No_Consumido) loop
         Apunt_T := new Tokens.Token;
         Tokens.Copiar (Apunt_T.all, Token_Reconocido);
         -- en la instruccion previa se encuentra la llamada
         -- a la funcion que realiza el analisis lexico (Token_Reconocido),
         -- y ahi se produce la modificacion de No_Consumido
         Listas_Apunt_Tokens.Annadir_Token (LAT, Apunt_T);
      end loop;
      --Apunt_T := new Tokens.Token;
      --Tokens.Copiar (Apunt_T.all, Token_Reconocido);
      -- para que reconozca el token Fin_Linea
      --Listas_Apunt_Tokens.Annadir_Token (LAT, Apunt_T);
      -- token Fin_Linea annadido al final de la lista
   end Obtener_Lista_Apunt_Tokens;

end Analexico;

