with strings_variables,
     listas_apunt_tokens,
     analexico,
     tokens;

package lineas is

     type Linea is private;

     function Crear (NumLinea: Positive;
                     Str: Strings_Variables.String_Variable) return Linea;
     -- Entrada: Positivo <- NumLinea
     --          String_Variable <- Str
     -- Salida : Devuelve un objeto de tipo Linea
     -- Efecto : Construye un objeto de tipo linea a partir de 'Str' y
     --          'NumLinea', se procede al analisis lexico de 'str' y la
     --          lista de apuntadores de tokens resultante se asigna al
     --          campo analisis.

     function Numero_Linea (Lin: Linea) return Positive;
     -- Entrada: Linea <- Lin
     -- Salida : devuelve un entero positivo
     -- Efecto : Devuelve el campo 'NumLinea' del objeto de tipo 'Linea'
     --          que pasemos como parametro

     function Cadena (Lin: Linea) return Strings_Variables.String_Variable;
     -- Entrada: Linea <- Lin
     -- Salida : devuelve un string_variable
     -- Efecto : construye un string_variable a partir del campo 'analisis'
     --          del parametro 'Lin'

     function Analisis (Lin: Linea)
          return Listas_Apunt_Tokens.Tipo_Lista_Apunt_Tokens;
     -- Entrada: Linea <- Lin
     -- Salida : devuelve un objeto de tipo 'tipo_lista_apunt_tokens'
     -- Efecto : devuelve el campo 'analisis' del parametro 'Lin'

     procedure Copiar (L1: out Linea; L2: in Linea);
     -- Entrada: Linea <- L1,L2
     -- Salida : Linea -> L1
     -- Efecto : Copia L2 sobre L1

     procedure Copiar_Lista (L1: out Linea; L2: in Linea);
     -- Entrada: Linea <- L1,L2
     -- Salida : Linea -> L2
     -- Efecto : Copia L2 sobre L1 haciendo una nueva copia de la lista
     --          de apuntadores de tokens del campo 'analisis' de L2
     --          sobre el campo 'analisis' de L2

     procedure Traducir (Lin  : in out Linea;
                         Tok  : in Tokens.Token;
                         Trad : in Strings_Variables.String_Variable);
     -- Entrada: Linea <- Lin
     --          Token <- Tok
     --          String_variable <- Trad
     -- Salida : Linea -> Lin (linea traducida)
     -- Efecto : Busca el token 'tok' en la lista de apuntadores de tokens
     --          del campo 'analisis' de lin y lo traduce 'Trad'

     Error_Lexico : Exception;
     -- se ha producido un error lexico en el procedimiento crear durante
     -- la aperacion 'Analexico.Obtener_Lista_Apunt_Tokens'


private

     type Linea is
        record
           Analisis : Listas_Apunt_Tokens.Tipo_Lista_Apunt_Tokens;
           NumLinea : Positive;
        end record;


end lineas;
