with Equivalencias,
     Strings_Variables,
     Listas_Ordenadas;

package lista_equivalencias is

     procedure Inicializar;
     -- Entrada:
     -- Salida :
     -- Efecto : inicializa la lista de equivalencias

     procedure Cargar_Fichero (Nombre: in String);
     -- Entrada: String <- Nombre
     -- Salida :
     -- Efecto : construye una nueva lista de equivalencias cargando las
     --          equivalencias de un fichero de texto 'Nombre' si no se
     --          encuentra el fichero de elevara 'Fichero_No_Encontrado'
     --          si el fichero tiene un formato incorrecto o esta incom-
     --          pleto se elevara 'Fichero_Incorrecto' el formato del
     --          fichero ha de ser:
     --
     --      linea n ->          [ Identificador
     --      linea n+1 ->          (Traduccion)  ]

     procedure Guardar_Fichero (Nombre: in String);
     -- Entrada: String <- Nombre
     -- Salida :
     -- Efecto : guarda la lista de equivalencias actual en un fichero
     --          de texto de nombre 'Nombre'

     procedure Annadir(E: in Equivalencias.Equivalencia);
     -- Entrada: Equivalencia <- E
     -- Salida :
     -- Efecto : aade la equivalencia E a la lista de equivalencias

     procedure Eliminar(E: in Equivalencias.Equivalencia);
     -- Entrada: Equivalencia <- E
     -- Salida :
     -- Efecto : elimina la equivalencia E de la lista de equivalencias

     function Esta(E: Strings_Variables.String_Variable) return Boolean;
     -- Entrada: String_Variable <- E
     -- Salida : devuelve un objeto de tipo booleano
     -- Efecto : si existe una equivalencia en la lista de equivalencias cuya
     --          clave sea E devuelve true, false c.c

     function Traduccion(Orig: Strings_Variables.String_Variable)
          return Strings_Variables.String_Variable;
     -- Entrada: String_Variable <- Orig
     -- Salida : devuelve un String_Variable
     -- Efecto : si existe una equivalencia en la lista de equivalencias
     --          cuya clave sea Orig devuelve la traduccion de dicha
     --          equivalencia


     Fichero_No_encontrado : exception;
     Fichero_Incorrecto    : exception;


private

     subtype StrClave is String(1..40);

     function DameClave (E: Equivalencias.Equivalencia) return StrClave;
     -- Dada una equivalencia devuelve la clave para su insercin en
     -- la lista de equivalencias

     package Equivalencias_Ordenadas is new Listas_Ordenadas(
               Elemento => Equivalencias.Equivalencia,
               TClave   => StrClave,
               Clave    => DameClave,
               Copiar   => Equivalencias.Copiar,
               Orden    => "<" );


end lista_equivalencias;
