generic
    type Elemento is private;
package Listas is

        type Lista is private;

        procedure Crear_Vacia (L : out Lista);
        -- Entrada:
        -- Salida : Lista -> L
        -- Efecto : Crea una lista vacia L

        procedure Annadir (L : in out Lista; E : Elemento);
        -- Entrada: Lista <- L
        --          Elemento <- E
        -- Salida : Lista -> L
        -- Efecto : Crea una nueva lista L con el elemento 'E' aadido
        --          al final de la lista

        procedure Insertar (L : in out Lista; E : Elemento; N : Positive);
        -- Entrada: Lista <- L
        --          Elemento <- E
        --          Positivo <- N
        -- Salida : Lista -> L
        -- Efecto : Crea una nueva lista L con el elemento 'E' aadido
        --          en la N-esima posicion de la lista, si no es posible
        --          se eleva la excepcion 'Posicion_Excesiva'

        function Valor (L : in Lista; N : in Positive) return Elemento;
        -- Entrada: Lista <- L
        --          Positivo <- N
        -- Salida : devuelve un objeto de tipo elemento
        -- Efecto : devueleve el elemento del nodo N-esimo de la lista L, si
        --          no es posible se eleva 'Posicion_Excesiva'

        function Esta (L : Lista; E : Elemento) return Boolean;
        -- Entrada: Lista <- L
        --          Elemento <- E
        -- Salida : devuelve un booleano
        -- Efecto : si el elemento E esta en la lista devuelve 'true' en caso
        --          contrario devuelve 'false'

        procedure Eliminar (L : in out Lista; E : Elemento);
        -- Entrada: Lista <- L
        --          Elemento <- E
        -- Salida : Lista -> L
        -- Efecto : crea una nueva lista 'L' de la que se ha eleiminado el
        --          nodo que contenia el elemento 'E'

        function Longitud (L : Lista) return Natural;
        -- Entrada: Lista <- L
        -- Salida : devuelve un natural con la longitud de la lista
        -- Efecto :


        function Primero (L : Lista) return Elemento;
        -- Entrada: Lista <- L
        -- Salida : Elemento -> E
        -- Efecto : devuelve el elemento del primer nodo de la lista

        function Ultimo  (L : Lista) return Elemento;
        -- Entrada: Lista <- L
        -- Salida : Elemento -> E
        -- Efecto : devuelve el elemento del ultimo nodo de la lista

        function Resto (L : Lista) return Lista;
        -- Entrada: Lista <- L
        -- Salida : devuelve objeto de tipo Lista
        -- Efecto : elimina el primer nodo de la lista L y devueleve la
        --          nueva lista

        function Principal (L : Lista) return Lista;
        -- Entrada: Lista <- L
        -- Salida : devueleve un objeto de tipo Lista
        -- Efecto : elimina el ultimo nodo de la lista L y devuelve la
        --          nueva lista


        function Posicion (L :  Lista; E : Elemento) return Positive;
        -- Entrada: Lista <- L
        --          Elemento <- E
        -- Salida : devuelve un entero positivo que indica la posicion del
        --          elemento E dentro de la lista
        -- Efecto : en caso de que el elemento E no se encuentre en la lista
        --          L se eleva la excepcin 'Elemento_Inexistente'

        function Es_Vacia (L : Lista) return Boolean;
        -- Entrada: Lista <- L
        -- Salida : devuelve un booleano que indica si la lista es vacia o no
        -- Efecto :

        procedure Copiar (L2 : out Lista; L1 : in Lista);
        -- Entrada: Lista <- L1
        -- Salida : Lista -> L2
        -- Efecto : L2 apunta al mismo nodo que L1

        procedure Copiar_Lista (L2 : out Lista; L1 : in Lista);
        -- Entrada: Lista <- L1
        -- Salida : Lista -> L2
        -- Efecto : Se crea un nueva lista L2 que es una copia nodo a nodo
        --          de la lista L1

        Lista_Vacia : exception;
        Posicion_Excesiva: exception;
        Elemento_Inexistente: exception;

private
        type Nodo;
        type Lista is access Nodo;  --implementacion dinamica
                                    --lista simple
        type Nodo is
            record
                 Info : Elemento;
                 Sig : Lista;
            end record;
end Listas;

