-- *---------------------------------------------------------------------*
-- |                                                                     |
-- |    Practica de programacin modular.                                |
-- |    Programa para traducir programas escritos en ADA.                |
-- |                                                                     |
-- |     LISTORD.ADS                                                     |
-- |       Especificacion del paquete generico Listas_Ordenadas          |
-- |                                                                     |
-- |     12-Mayo-1996                                                    |
-- |                                                                     |
-- *---------------------------------------------------------------------*


generic

  type Elemento is limited private;
  -- Indica el tipo de elemento que va a formar la lista
  type TClave is private;
  -- Indica el tipo de clave de ordenacin de la lista
  with function Orden (E1, E2: TClave) return Boolean;
  -- funcion para efectuar la ordenacin de los elementos de la lista
  with function Clave (E : Elemento) return TClave;
  -- a partir de un Elemento devuelve la clave de ordenacin
  with procedure Copiar (E1: out Elemento; E2: in Elemento);
  -- copia dos elementos

package listas_ordenadas is

     type ListOrd is limited private;

     procedure Crear_Vacia (L: out ListOrd);
     -- Entrada:
     -- Salida : ListOrd -> L
     -- Efecto : crea una lista ordenada vacia L


     procedure Insertar (L: in out ListOrd; E: Elemento);
     -- Entrada: ListOrd <- ListOrd
     --          Elemento <- E
     -- Salida : ListOrd -> L
     -- Efecto : Inserta el elemento E en la lista L en la posicin adecuada
     --          para mantener la lista ordenada

     function Buscar (L: in ListOrd; K: in TClave) return Elemento;
     -- Entrada: ListOrd <- L
     --          TClave <- k
     -- Salida : devuelve un objeto de tipo Elemento
     -- Efecto : busca en la lista L el elemento de clave K y lo devuelve
     --          en caso de que no halla ningun elemento con esa clave se
     --          elevar la excepcin 'Inexistente'

     function Esta (L: ListOrd; E: Elemento) return Boolean;
     -- Entrada: ListOrd <- L
     --          Elemento <- E
     -- Salida : devuelve un objeto de tipo booleano
     -- Efecto : si el elemento E esta en la lista L devuelve true, en
     --          caso contrario devuelve false

     function Primero (L: ListOrd) return Elemento;
     -- Entrada: ListOrd <- L
     -- Salida : devuelve un objeto de tipo Elemento
     -- Efecto : devuelve el elemento contenido en el primer nodo de la
     --          lista L

     function Resto (L: ListOrd) return ListOrd;
     -- Entrada: ListOrd <- L
     -- Salida : devuelve un objeto de tipo ListOrd
     -- Efecto : devuelve una copia de lista L a la que se ha extraido
     --          su primer elemento

     procedure Eliminar (L: in out ListOrd; E: Elemento);
     -- Entrada: ListOrd <- L
     --          Elemento <- E
     -- Salida : ListOrd -> L
     -- Efecto : Elimina de la lista L el nodo que contenga el elemento E
     --

     function Es_Vacia (L: ListOrd) return Boolean;
     -- Entrada: ListOrd <- L
     -- Salida : objeto de tipo booleano
     -- Efecto : devuelve true si la lista L est vacia, false c.c

     procedure Copiar (L1: out ListOrd; L2: in ListOrd);
     -- Entrada: ListOrd <- L2
     -- Salida : ListOrd -> L1
     -- Efecto : L1 apunta a la misma lista que L2

     Lista_Vacia : exception;
     Inexistente : exception;

private

     type Nodo;
     type ListOrd is access Nodo;

     type Nodo is
          record
               Info : Elemento;
               Siguiente : ListOrd;
          end record;


end listas_ordenadas;

