package Strings_Variables is

   Max : constant Positive := 100;
   subtype Indice is Natural range 0 .. Max;
   type String_Variable is limited private;


   function FString_Var (S : in String) return String_Variable;
   -- Entrada: String <- S
   -- Salida : devuelve un objeto de tipo String_Variable
   -- Efecto : construye un string_variable a partir del string S

   function FString_Vacio return String_Variable;
   -- Entrada:
   -- Salida : devuelve un objeto de tipo string_variable
   -- Efecto : construye un string_variable vacio, osea, de longitud cero

   function Longitud (T : in String_Variable) return Indice;
   -- Entrada: String_Variable <- T
   -- Salida : devuelve un objeto de tipo Indice
   -- Efecto : devuelve la longitud, numero de caracteres que contiene,
   --          del string_variable T

   function Valor (T : in String_Variable) return String;
   -- Entrada: String_Variable <- T
   -- Salida : devuelve un objeto de tipo String
   -- Efecto : dado un string_variable, se devuelve la cadena de
   --          caracteres (string) que contiene

   function Es_Vacio (T : in String_Variable) return Boolean;
   -- Entrada: String_variable <- T
   -- Salida : devuelve un objeto de tipo boolean
   -- Efecto : si el string_variable T esta vacio (tiene longitud cero),
   --          se devuelve true, en caso contrario false

   function "&" (T1, T2 : in String_Variable) return String_Variable;
   -- Entrada: String_variable <- T1
   --          String_variable <- T2
   -- Salida : devuelve un objeto de tipo string_variable
   -- Efecto : devuelve un nuevo string_variable resultado de concatenar
   --          el valor de T1 con el valor de T2 y cuya longitud sera la
   --          suma de la longitud de T1 y la longitud de T2

   function "&" (T : in String_Variable; S : in String) return String_Variable;
   -- Entrada: String_Variable <- T
   --          String <- S
   -- Salida : devuelve un objeto de tipo string_variable
   -- Efecto : construye un nuevo string_variable resultado de concatenar el
   --          valor de T con el string S cuya longitud sera la suma de la
   --          longitud de T y el tamao del string S

   function "&" (S : in String; T : in String_Variable) return String_Variable;
   -- Entrada: String <- S
   --          String_Variable <- T
   -- Salida : devuelve un objeto de tipo String_Variable
   -- Efecto : se construye un string_variable resultado de concatenar el
   --          string S con valor del string_variable T y cuya longitud sera
   --          la suma del tamao del string S con la longitud del
   --          string_variable T

   function "=" (T1, T2 : in String_Variable) return Boolean;
   -- Entrada: String_Variable <- T1
   --          String_Variable <- T2
   -- Salida : devuelve un onjetoo de tipo boolean
   -- Efecto : si el valor de T1 es igual al valor de T2 y tienen la
   --          misma longitud devuelve true y false en caso contrario.

   function "<=" (T1, T2 : in String_Variable) return Boolean;
   -- Entrada: String_Variable <- T1
   --          String_Variable <- T2
   -- Salida : devuelve un objeto de tipo boolean
   -- Efecto : si el valor de T1 es menor o igual que el valor de T2 y
   --          la longitud de T1 es menor o igual que la longitud de T2
   --          devuelve true, si no es asi devuelve false

   function ">=" (T1, T2 : in String_Variable) return Boolean;
   -- Entrada: String_Variable <- T1
   --          String_Variable <- T2
   -- Salida : devuelve un objeto de tipo boolean
   -- Efecto : si el valor de T1 es mayor o igual que el valor de T2 y
   --          ademas la longitud de T1 es mayor o igual que longitud de
   --          T2 se devuelve true, en caso contrario sera false

   function "<" (T1, T2 : in String_Variable) return Boolean;
   -- Entrada: String_Variable <- T1
   --          String_Variable <- T2
   -- Salida : se devuelve un objeto de tipo booleano
   -- Efecto : si el valor de T1 es menor que el valor de T2 y ademas la
   --          la longitud de T1 es menor que la longitud de T2 se devuelve
   --          true y false si no es asi

   function ">" (T1, T2 : in String_Variable) return Boolean;
   -- Entrada: String_Variable <- T1
   --          String_Variable <- T2
   -- Salida : se devuelve un objeto de tipo boolean
   -- Efecto : si el valor de T1 es mayor que el valor de T2 y se cumple
   --          que la longitud de T1 es mayor que la de T2 se devuelve true
   --          de no ser asi se devuelve false

   procedure Copiar (T1 : out String_Variable; T2 : in String_Variable);
   -- Entrada: String_Variable <- T2
   -- Salida : String_Variable -> T1
   -- Efecto : T1 toma el mismo valor y longitud que tenga T2

   function En_Minusculas (T : in String_Variable) return String_Variable;
   -- Entrada: String_Variable <- T
   -- Salida : devuelve un objeto de tipo string_variable
   -- Efecto : se construye un nuevo string_variable de la misma longitud de
   --          T y en cuyo valor se sustituyen todos los caracteres alfabeticos
   --          que se encuentren en mayusculas por su correspondiente caracter
   --          en minusculas

private

   type String_Variable is
      record
         Long : Indice := 0;
         Valor : String (1 .. Max);
      end record;


end Strings_Variables;
