with no_interactivo,
     interactivo,
     strings_variables,
     gestion_menus,
     text_io;

use interactivo,
    no_interactivo;

procedure Traduktor is


 -- Entrada: Positivo <- Codigo ( codigo de error )
 -- Salida :
 -- Efecto : Escribe por pantalla un mensaje de error correspondiente al
 --          codigo de error especificado
 --
 --          Codigo     Excepcion
 --          ------     --------------------------------
 --             1       Error_Lexico
 --             2       Identificadores_No_Encontrado
 --             3       Palabras_Reservadas_No_Encontrado
 --             4       Equivalencias_No_Encontrado
 --             5       Equivalencias_Incorrecto
 --             6       Storage_Error
 --             7       Original_No_Encontrado
 --             8       Error no determinado

 procedure Mensaje_Error (Codigo : Positive) is
   begin
     Text_io.Put (ASCII.ESC & "[2J");  -- borrar pantalla
     Text_io.Put (">>> ERROR: ");
   case Codigo is
        when 1 =>
           Text_io.Put_Line ("El programa introducido " &
                             "NO ES un programa ADA correcto.");

        when 2 =>
           Text_io.Put_Line ("No se ha encontrado el fichero "&
                             "de identificadores estandar.");

        when 3 =>
           Text_io.Put_Line ("No se ha encontrado el fichero "&
                             "de palabras reservadas de ADA.");

        when 4 =>
           Text_io.Put_Line ("No se ha encontrado el fichero " &
                             "de equivalencias que usted especifico.");

        when 5 =>
           Text_io.Put_Line ("El fichero de equivalencias que " &
                             "usted especifico no es valido, reviselo.");

        when 6 =>
            Text_io.Put_Line ("No hay suficiente memoria libre " &
                              "para ejecutar el programa.");

        when 7 =>
           Text_io.Put_Line ("No se ha encontrado el fichero " &
                             "de entrada que usted especifico.");

        when  8 =>
           Text_io.Put_Line ("NO DETERMINADO <<<");
           Text_io.Put_Line ("Pongase en contacto con el servivio tecnico.");
        when Others =>
           null;
      end case;
      Text_io.Put ("Pulse <ENTER> para continuar...");
      Text_io.Skip_Line;
    end Mensaje_Error;



    function LeerNombre return Strings_Variables.String_Variable is
         Str : String(1..60) := (others => ' ');
         Lon : Natural;
    begin
      Text_io.Get_Line (Str,Lon);
      return Strings_Variables.FString_Var(Str(1..Lon));
    end LeerNombre;


    procedure Menu_Interactivo is
        Visualizable : Boolean := True;
        MP : Gestion_Menus.TMenu;
        Comentarios : Boolean := False;
        Fich_Orig, Fich_Trad : Strings_Variables.String_Variable;


        procedure Hacer_Traduccion (Comentarios : in Boolean) is
          begin
            Text_io.Put (ASCII.ESC & "[2J"); -- borrar pantalla
            Text_io.Put ("Introduce nombre fichero de entrada: ");
            Strings_Variables.Copiar (Fich_Orig, LeerNombre);
            Text_io.Put ("Introduce nombre fichero de salida: ");
            Strings_Variables.Copiar (Fich_Trad, LeerNombre);
            Interactivo.Inicializar (Strings_Variables.Valor(Fich_Orig));
            Interactivo.Traducir (Comentarios);
            Interactivo.Terminar (Strings_Variables.Valor(Fich_Trad));
          end Hacer_Traduccion;


    begin
        while Visualizable loop
            Gestion_Menus.Inicializar ("interactivo.mnu", MP);
            Gestion_Menus.Visualizar (MP);
            case Gestion_Menus.Opcion is
               when '0' => Visualizable := False;
               when '1' => Hacer_Traduccion (False);
               when '2' => Hacer_Traduccion (True);
               when others => null;
            end case;
        end loop;
    exception
        when Interactivo.Palabras_Reservadas_No_Encontrado =>
          Mensaje_Error (3);
        when Interactivo.Identificadores_No_Encontrado =>
          Mensaje_Error (2);
        when Interactivo.Original_No_Encontrado =>
          Mensaje_Error (7);
        when Interactivo.Error_Lexico =>
          Mensaje_Error (1);
        when Storage_Error =>
          Mensaje_Error (6);
        when Others =>
          Mensaje_Error (8);
    end Menu_Interactivo;



    procedure Menu_NoInteractivo is
        Visualizable : Boolean := True;
        MP : Gestion_Menus.TMenu;
        Original,
        Traducido,
        Identificadores : Strings_Variables.String_Variable;
    begin
        while Visualizable loop
            Gestion_Menus.Inicializar ("no_interactivo.mnu", MP);
            Gestion_Menus.Visualizar (MP);
            case Gestion_Menus.Opcion is
               when '0' => Visualizable := False;
               when '1' =>
                 Text_io.Put (ASCII.ESC & "[2J"); -- borrar pantalla
                 Text_io.Put ("Introduce nombre fichero de entrada: ");
                 Strings_Variables.Copiar (Original, LeerNombre);
                 Text_io.Put ("Introduce nombre fichero de identificadores: ");
                 Strings_Variables.Copiar (Identificadores, LeerNombre);
                 No_Interactivo.Crear_Listado_Identificadores (
                        Strings_Variables.Valor (Original),
                        Strings_Variables.Valor (Identificadores));
               when '2' =>
                 Text_io.Put (ASCII.ESC & "[2J"); -- borrar pantalla
                 Text_io.Put ("Introduce nombre fichero de entrada: ");
                 Strings_Variables.Copiar (Original, LeerNombre);
                 Text_io.Put ("Introduce nombre fichero de salida: ");
                 Strings_Variables.Copiar (Traducido, LeerNombre);
                 Text_io.Put ("Introduce nombre fichero de identificadores: ");
                 Strings_Variables.Copiar (Identificadores, LeerNombre);
                 No_Interactivo.Traducir (
                      Strings_Variables.Valor (Original),
                      Strings_Variables.Valor (Identificadores));
                 No_Interactivo.Terminar (Strings_Variables.Valor (Traducido));
               when others => null;
            end case;
        end loop;
    exception
        when No_Interactivo.Palabras_Reservadas_No_Encontrado =>
          Mensaje_Error (3);
        when No_Interactivo.Identificadores_No_Encontrado =>
          Mensaje_Error (2);
        when No_Interactivo.Original_No_Encontrado =>
          Mensaje_Error (7);
        when No_Interactivo.Equivalencias_No_Encontrado =>
          Mensaje_Error (4);
        when No_Interactivo.Equivalencias_Incorrecto =>
          Mensaje_Error (5);
        when No_Interactivo.Error_Lexico =>
          Mensaje_Error (1);
        when Storage_Error =>
          Mensaje_Error (6);
        when Others =>
          Mensaje_Error (8);
    end Menu_NoInteractivo;



    procedure Menu_Principal is
        Visualizable : Boolean := True;
        MP : Gestion_Menus.TMenu;
    begin
        while Visualizable loop
            Gestion_Menus.Inicializar ("principal.mnu", MP);
            Gestion_Menus.Visualizar (MP);
            case Gestion_Menus.Opcion is
               when '0' => Visualizable := False;
               when '1' => Menu_Interactivo;
               when '2' => Menu_NoInteractivo;
               when others => null;
            end case;
        end loop;
    end Menu_Principal;



 begin
   Menu_Principal;
 end traduktor;

